#!/bin/sh

CP="/bin/cp"
LN="/bin/ln"
SED="/bin/sed"
MKDIR="/bin/mkdir"
DIRNAME="/usr/bin/dirname"
LD_LINUX="/lib/ld-linux.so.3"

INIT="/usr/lib/initrd/init.sh"

INITRD_ROOT=/mnt/initrd

OBJECTS_SPECIFY_LIST="/usr/lib/initrd/initrd-file.list"

#-----------------------------------------------------------------------------
#  show_help()
#-----------------------------------------------------------------------------
show_help() {
    echo
    echo "usage: `basename $0`"
    echo "make initrd image to $INITRD_ROOT"
    echo
}

#-----------------------------------------------------------------------------
#       Prepare parent directory
#-----------------------------------------------------------------------------
mkdir_p_parent() {
    dst_dir=`"$DIRNAME" "$1"`
    if [ ! -d "$dst_dir" -a ! -L "$dst_dir" ]; then
        "$MKDIR" -p "$dst_dir"
    fi
}

#-----------------------------------------------------------------------------
#       Copy content to root of mounted image
#-----------------------------------------------------------------------------
do_copy() {
    src=$1
    dst="$INITRD_ROOT/$src"

    if [ ! -e "$src" -o -e "$dst" ]; then
        return
    fi

    mkdir_p_parent $dst

    "$CP" -f "$src" "$dst"
}

#-----------------------------------------------------------------------------
#       Get dependency libraries
#-----------------------------------------------------------------------------
get_dep_libs() {
    "$LD_LINUX" --list $1 | "$SED" -r 's|^[^/]+([^ \(]+).*$|\1|'
}

#-----------------------------------------------------------------------------
#       Gather initrd objects
#-----------------------------------------------------------------------------
get_initrd_objects() {
    source "${OBJECTS_SPECIFY_LIST}"

    SYS_SKELDIR=$(echo "$SYS_SKELDIR" | sort | uniq)
    DIR_LINKS=$(echo "$DIR_LINKS" | sort | uniq)
    PROGS=$(echo "$PROGS" | sort | uniq)
    SYM_LINKS=$(echo "$SYM_LINKS" | sort | uniq)
}

#-----------------------------------------------------------------------------
#       Prepare directory objects
#-----------------------------------------------------------------------------
prepare_directory_objects() {
    for dir in $@; do
        "$MKDIR" -p "$INITRD_ROOT$dir"
    done
}

#-----------------------------------------------------------------------------
#       Copy program objects
#-----------------------------------------------------------------------------
program_objects() {
    for content in $@; do
        do_copy $content

        DEP_LIBS=$(get_dep_libs $content)
        for lib in $DEP_LIBS; do
            do_copy $lib
        done
    done
}

#-----------------------------------------------------------------------------
#       Copy symlink objects
#-----------------------------------------------------------------------------
symlink_objects() {
    for i in $@; do
        if [ "z$i" == "z" ]; then
            continue
        fi

        link=${i%:*}
        target=${i#*:}
        mkdir_p_parent "$INITRD_ROOT$link"
        "$LN" -s "$target" "$INITRD_ROOT$link"
    done
}

#-----------------------------------------------------------------------------
#       Copy content
#-----------------------------------------------------------------------------
make_initrd() {
    echo "copy initrd objects to $INITRD_ROOT"
    get_initrd_objects

    prepare_directory_objects $SYS_SKELDIR
    symlink_objects $DIR_LINKS
    program_objects $PROGS
    symlink_objects $SYM_LINKS

    "$CP" -f "$INIT" "$INITRD_ROOT/sbin/init"
}

#-----------------------------------------------------------------------------
#       Check given parameter is mount point
#-----------------------------------------------------------------------------
check_mount_point() {
    grep " $1 " /etc/mtab > /dev/null
    if [ $? -ne 0 ]; then
        echo "$1 is not a mount point"
        exit 0
    fi
}

#-----------------------------------------------------------------------------
#       Main
#-----------------------------------------------------------------------------

if [ $# -ge 1 ]; then
    echo "Error: too many arguments were given."
    show_help >&2
    exit 1
fi

check_mount_point $INITRD_ROOT
make_initrd
